
{ NAME: antinag.pas                                                         }
{ VERSION: 1.0                                                              }
{ DATE: September 1994.                                                     }
{ DESCRIPTION: Try to decode NAGRAVISON images.                             }
{ ENVIRONMENT: Turbo Pascal 7.0                                             }

program AntiNagravision;

uses crt;

const
  max_res=512;      { Maximun resolution for PGM file }

type
  tbuffer=array [1..max_res] of byte;
  tlines=array [1..max_res] of word;
  pgm_header=record
    x_max,y_max,gray_scale,len:word;
  end;

var
  buffer:tbuffer;
  sort_lines:tlines;
  head:pgm_header;
  f_src,f_dest:file;
  head_ok:boolean;

{--------------------------------------------------------------------------}
procedure procces_header(var f_src,f_dest:file;var head:pgm_header;
                          var buffer:tbuffer;var head_ok:boolean);
var
  w,code:word;
  str_aux:string;

procedure get_text(var buffer:tbuffer;var i:word;var out:string);
begin
  out:='';
  inc(i);
  while buffer[i]<>$0a do
    begin
      out:=out+chr(buffer[i]);
      inc(i);
    end;
end;

begin
  writeln('- Proccesing header...');
  blockread(f_src,buffer,max_res);
  head.len:=0;
  get_text(buffer,head.len,str_aux);
  head_ok:=(str_aux='P5');
  if head_ok
    then
      begin
        get_text(buffer,head.len,str_aux);
        w:=pos(' ',str_aux);
        val(copy(str_aux,1,w-1),head.x_max,code);
        val(copy(str_aux,w+1,ord(str_aux[0])-w),head.y_max,code);
        get_text(buffer,head.len,str_aux);
        val(str_aux,head.gray_scale,w);
        blockwrite(f_dest,buffer,head.len);
        if (head.x_max>max_res) or (head.y_max>max_res)
          then
            head_ok:=false;
      end;
end;
{--------------------------------------------------------------------------}
procedure get_line(var f_src:file;var buffer:tbuffer;i:word;
                   var head:pgm_header;var sort_lines:tlines);
begin
  seek(f_src,head.len+(head.x_max*(sort_lines[i]-1)));
  blockread(f_src,buffer,head.x_max);
end;
{--------------------------------------------------------------------------}
procedure resort_lines(var f_src:file;var head:pgm_header;
                       var sort_lines:tlines;var buffer:tbuffer);
var
  l1,l2,i,line_max_cor:word;
  max_cor,cor:longint;
  pre_cor:array [1..max_res] of longint;
  x,y:byte;

procedure swap_lines(l1,l2:word;var sort_lines:tlines);
var
  aux:word;
begin
  aux:=sort_lines[l2];
  sort_lines[l2]:=sort_lines[l1];
  sort_lines[l1]:=aux;
end;

begin
  write('- Recorrelating lines... ');
  x:=wherex;
  y:=wherey;
  for i:=1 to head.y_max do sort_lines[i]:=i;
  for l1:=1 to (head.y_max-1) do
    begin
      gotoxy(x,y);
      write(head.y_max-1-l1:3);
      max_cor:=0;
      get_line(f_src,buffer,l1,head,sort_lines);
      for i:=1 to head.x_max do pre_cor[i]:=buffer[i]*i;
      for l2:=(l1+1) to head.y_max do
        begin
          get_line(f_src,buffer,l2,head,sort_lines);
          cor:=0;
          for i:=1 to head.x_max do cor:=cor+pre_cor[i]*buffer[i];
          if (cor>max_cor)
            then
              begin
                max_cor:=cor;
                line_max_cor:=l2;
              end;
        end;
      swap_lines(l1+1,line_max_cor,sort_lines);
    end;
  writeln;
end;
{--------------------------------------------------------------------------}
procedure save_lines(var f_src,f_dest:file;var head:pgm_header;
                     var sort_lines:tlines;var buffer:tbuffer);
var
  i:word;
begin
  writeln('- Saving correlated lines...');
  for i:=1 to head.y_max do
    begin
      get_line(f_src,buffer,i,head,sort_lines);
      blockwrite(f_dest,buffer,head.x_max);
    end;
end;
{--------------------------------------------------------------------------}
{--------------------------------------------------------------------------}
begin
  if (paramcount<>2)
    then
      writeln('ANTINAG file_src.pgm file_dest.pgm')
    else
      begin
        assign(f_src,paramstr(1));
        reset(f_src,1);
        assign(f_dest,paramstr(2));
        rewrite(f_dest,1);
        procces_header(f_src,f_dest,head,buffer,head_ok);
        if head_ok
          then
            begin
              resort_lines(f_src,head,sort_lines,buffer);
              save_lines(f_src,f_dest,head,sort_lines,buffer);
            end
          else
            writeln('-> Incorrect PGM header. <-');
        close(f_src);
        close(f_dest);
      end;
end.
